/*	$OpenBSD: s_lroundl.c,v 1.2 2012/12/05 23:20:04 deraadt Exp $	*/

/*-
 * Copyright (c) 2005 David Schultz <das@FreeBSD.ORG>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <limits.h>
#include <fenv.h>
#include <math.h>

#ifndef LROUND_TYPE
#if 0
__FBSDID("$FreeBSD: src/lib/msun/src/s_lround.c,v 1.2 2005/04/08 00:52:16 das Exp $");
#endif
#define LROUND_TYPE		long double
#define	LROUND_ROUNDIT		roundl
#define LROUND_DTYPE		long
#define	LROUND_DTYPE_MIN		LONG_MIN
#define	LROUND_DTYPE_MAX		LONG_MAX
#define	LROUND_FN		lroundl
#endif

/*
 * If LROUND_TYPE has more precision than LROUND_DTYPE, the endpoints LROUND_DTYPE_(min|max) are
 * of the form xxx.5; they are "out of range" because lround() rounds away
 * from 0.  On the other hand, if LROUND_TYPE has less precision than LROUND_DTYPE, then
 * all values that are out of range are integral, so we might as well assume
 * that everything is in range.  At compile time, INRANGE(x) should reduce to
 * two floating-point comparisons in the former case, or TRUE otherwise.
 */
static const LROUND_TYPE LROUND_DTYPE_min = LROUND_DTYPE_MIN - 0.5;
static const LROUND_TYPE LROUND_DTYPE_max = LROUND_DTYPE_MAX + 0.5;
#define	INRANGE(x)	(LROUND_DTYPE_max - LROUND_DTYPE_MAX != 0.5 || \
			 ((x) > LROUND_DTYPE_min && (x) < LROUND_DTYPE_max))

LROUND_DTYPE
LROUND_FN(LROUND_TYPE x)
{

	if (INRANGE(x)) {
		x = LROUND_ROUNDIT(x);
		return ((LROUND_DTYPE)x);
	} else {
		feraiseexcept(FE_INVALID);
		return (LROUND_DTYPE_MAX);
	}
}
